/* This file is part of the wvWare 2 project
   Copyright (C) 2001 S.R.Haque <srhaque@iee.org>
   Copyright (C) 2003 Werner Trobin <trobin@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "word95_helper.h"
#include "word95_generated.h"
#include "olestream.h"
#include "styles.h"

#include <iostream>

namespace wvWare
{

namespace Word95
{

namespace SPRM
{

typedef enum {
    sprmNoop = 0,
    sprmPIstd = 2,
    sprmPIstdPermute = 3,
    sprmPIncLvl = 4,
    sprmPJc = 5,
    sprmPFSideBySide = 6,
    sprmPFKeep = 7,
    sprmPFKeepFollow = 8,
    sprmPFPageBreakBefore = 9,
    sprmPBrcl = 10,
    sprmPBrcp = 11,
    sprmPAnld = 12,
    sprmPNLvlAnm = 13,
    sprmPFNoLineNumb = 14,
    sprmPChgTabsPapx = 15,
    sprmPDxaRight = 16,
    sprmPDxaLeft = 17,
    sprmPNest = 18,
    sprmPDxaLeft1 = 19,
    sprmPDyaLine = 20,
    sprmPDyaBefore = 21,
    sprmPDyaAfter = 22,
    sprmPChgTabs = 23,
    sprmPFInTable = 24,
    sprmPFTtp = 25,
    sprmPDxaAbs = 26,
    sprmPDyaAbs = 27,
    sprmPDxaWidth = 28,
    sprmPPc = 29,
    sprmPBrcTop10 = 30,
    sprmPBrcLeft10 = 31,
    sprmPBrcBottom10 = 32,
    sprmPBrcRight10 = 33,
    sprmPBrcBetween10 = 34,
    sprmPBrcBar10 = 35,
    sprmPFromText10 = 36,
    sprmPWr = 37,
    sprmPBrcTop = 38,
    sprmPBrcLeft = 39,
    sprmPBrcBottom = 40,
    sprmPBrcRight = 41,
    sprmPBrcBetween = 42,
    sprmPBrcBar = 43,
    sprmPFNoAutoHyph = 44,
    sprmPWHeightAbs = 45,
    sprmPDcs = 46,
    sprmPShd = 47,
    sprmPDyaFromText = 48,
    sprmPDxaFromText = 49,
    sprmPFLocked = 50,
    sprmPFWidowControl = 51,
    sprmPRuler = 52,
    sprmCFStrikeRM = 65,
    sprmCFRMark = 66,
    sprmCFFldVanish = 67,
    sprmCPicLocation = 68,
    sprmCIbstRMark = 69,
    sprmCDttmRMark = 70,
    sprmCFData = 71,
    sprmCRMReason = 72,
    sprmCChse = 73,
    sprmCSymbol = 74,
    sprmCFOle2 = 75,
    sprmCIstd = 80,
    sprmCIstdPermute = 81,
    sprmCDefault = 82,
    sprmCPlain = 83,
    sprmCFBold = 85,
    sprmCFItalic = 86,
    sprmCFStrike = 87,
    sprmCFOutline = 88,
    sprmCFShadow = 89,
    sprmCFSmallCaps = 90,
    sprmCFCaps = 91,
    sprmCFVanish = 92,
    sprmCFtc = 93,
    sprmCKul = 94,
    sprmCSizePos = 95,
    sprmCDxaSpace = 96,
    sprmCLid = 97,
    sprmCIco = 98,
    sprmCHps = 99,
    sprmCHpsInc = 100,
    sprmCHpsPos = 101,
    sprmCHpsPosAdj = 102,
    sprmCMajority = 103,
    sprmCIss = 104,
    sprmCHpsNew50 = 105,
    sprmCHpsInc1 = 106,
    sprmCHpsKern = 107,
    sprmCMajority50 = 108,
    sprmCHpsMul = 109,
    sprmCCondHyhen = 110,
    sprmCFSpec = 117,
    sprmCFObj = 118,
    sprmPicBrcl = 119,
    sprmPicScale = 120,
    sprmPicBrcTop = 121,
    sprmPicBrcLeft = 122,
    sprmPicBrcBottom = 123,
    sprmPicBrcRight = 124,
    sprmSScnsPgn = 131,
    sprmSiHeadingPgn = 132,
    sprmSOlstAnm = 133,
    sprmSDxaColWidth = 136,
    sprmSDxaColSpacing = 137,
    sprmSFEvenlySpaced = 138,
    sprmSFProtected = 139,
    sprmSDmBinFirst = 140,
    sprmSDmBinOther = 141,
    sprmSBkc = 142,
    sprmSFTitlePage = 143,
    sprmSCcolumns = 144,
    sprmSDxaColumns = 145,
    sprmSFAutoPgn = 146,
    sprmSNfcPgn = 147,
    sprmSDyaPgn = 148,
    sprmSDxaPgn = 149,
    sprmSFPgnRestart = 150,
    sprmSFEndnote = 151,
    sprmSLnc = 152,
    sprmSGprfIhdt = 153,
    sprmSNLnnMod = 154,
    sprmSDxaLnn = 155,
    sprmSDyaHdrTop = 156,
    sprmSDyaHdrBottom = 157,
    sprmSLBetween = 158,
    sprmSVjc = 159,
    sprmSLnnMin = 160,
    sprmSPgnStart = 161,
    sprmSBOrientation = 162,
    sprmSBCustomize = 163,
    sprmSXaPage = 164,
    sprmSYaPage = 165,
    sprmSDxaLeft = 166,
    sprmSDxaRight = 167,
    sprmSDyaTop = 168,
    sprmSDyaBottom = 169,
    sprmSDzaGutter = 170,
    sprmSDMPaperReq = 171,
    sprmTJc = 182,
    sprmTDxaLeft = 183,
    sprmTDxaGapHalf = 184,
    sprmTFCantSplit = 185,
    sprmTTableHeader = 186,
    sprmTTableBorders = 187,
    sprmTDefTable10 = 188,
    sprmTDyaRowHeight = 189,
    sprmTDefTable = 190,
    sprmTDefTableShd = 191,
    sprmTTlp = 192,
    sprmTSetBrc = 193,
    sprmTInsert = 194,
    sprmTDelete = 195,
    sprmTDxaCol = 196,
    sprmTMerge = 197,
    sprmTSplit = 198,
    sprmTSetBrc10 = 199,
    sprmTSetShd = 200
} opcodes;

// The length of the SPRM parameter
U16 determineParameterLength( U8 sprm, const U8* in )
{
    switch ( sprm ) {
        case SPRM::sprmNoop:
            return 0;
        case SPRM::sprmPIstd:
            return 2;
        case SPRM::sprmPIstdPermute:
            break; // Variable.
        case SPRM::sprmPIncLvl:
        case SPRM::sprmPJc:
        case SPRM::sprmPFSideBySide:
        case SPRM::sprmPFKeep:
        case SPRM::sprmPFKeepFollow:
        case SPRM::sprmPFPageBreakBefore:
        case SPRM::sprmPBrcl:
        case SPRM::sprmPBrcp:
            return 1;
        case SPRM::sprmPAnld:
            break; // Variable.
        case SPRM::sprmPNLvlAnm:
        case SPRM::sprmPFNoLineNumb:
            return 1;
        case SPRM::sprmPChgTabsPapx:
            break; // Variable.
        case SPRM::sprmPDxaRight:
            return 2;
        case SPRM::sprmPDxaLeft:
        case SPRM::sprmPNest:
        case SPRM::sprmPDxaLeft1:
            return 2;
        case SPRM::sprmPDyaLine:
            return 4;
        case SPRM::sprmPDyaBefore:
        case SPRM::sprmPDyaAfter:
            return 2;
        case SPRM::sprmPChgTabs:
            if ( *in == 255 ) {
                U8 itbdDelMax = in[1];
                U8 itbdAddMax = in[1 + itbdDelMax * 4];
                return 1 + itbdDelMax * 4 + itbdAddMax * 3;
            }
            else
                return *in + 1;
        case SPRM::sprmPFInTable:
        case SPRM::sprmPFTtp:
            return 1;
        case SPRM::sprmPDxaAbs:
        case SPRM::sprmPDyaAbs:
        case SPRM::sprmPDxaWidth:
            return 2;
        case SPRM::sprmPPc:
            return 1;
        case SPRM::sprmPBrcTop10:
        case SPRM::sprmPBrcLeft10:
        case SPRM::sprmPBrcBottom10:
        case SPRM::sprmPBrcRight10:
        case SPRM::sprmPBrcBetween10:
        case SPRM::sprmPBrcBar10:
        case SPRM::sprmPFromText10:
            return 2;
        case SPRM::sprmPWr:
            return 1;
        case SPRM::sprmPBrcTop:
        case SPRM::sprmPBrcLeft:
        case SPRM::sprmPBrcBottom:
        case SPRM::sprmPBrcRight:
        case SPRM::sprmPBrcBetween:
        case SPRM::sprmPBrcBar:
            return 2;
        case SPRM::sprmPFNoAutoHyph:
            return 1;
        case SPRM::sprmPWHeightAbs:
        case SPRM::sprmPDcs:
        case SPRM::sprmPShd:
        case SPRM::sprmPDyaFromText:
        case SPRM::sprmPDxaFromText:
            return 2;
        case SPRM::sprmPFLocked:
        case SPRM::sprmPFWidowControl:
            return 1;
        case SPRM::sprmPRuler:
            break; // Variable.
        case SPRM::sprmCFStrikeRM:
        case SPRM::sprmCFRMark:
        case SPRM::sprmCFFldVanish:
            return 1;
        case SPRM::sprmCPicLocation:
            break; // Variable.
        case SPRM::sprmCIbstRMark:
            return 2;
        case SPRM::sprmCDttmRMark:
            return 4;
        case SPRM::sprmCFData:
            return 1;
        case SPRM::sprmCRMReason:
            return 2;
        case SPRM::sprmCChse:
            return 3;
        case SPRM::sprmCSymbol:
            break; // Variable.
        case SPRM::sprmCFOle2:
            return 1;
        case SPRM::sprmCIstd:
            return 2;
        case SPRM::sprmCIstdPermute:
        case SPRM::sprmCDefault:
            break; // Variable.
        case SPRM::sprmCPlain:
            return 0;
        case SPRM::sprmCFBold:
        case SPRM::sprmCFItalic:
        case SPRM::sprmCFStrike:
        case SPRM::sprmCFOutline:
        case SPRM::sprmCFShadow:
        case SPRM::sprmCFSmallCaps:
        case SPRM::sprmCFCaps:
        case SPRM::sprmCFVanish:
            return 1;
        case SPRM::sprmCFtc:
            return 2;
        case SPRM::sprmCKul:
            return 1;
        case SPRM::sprmCSizePos:
            return 3;
        case SPRM::sprmCDxaSpace:
        case SPRM::sprmCLid:
            return 2;
        case SPRM::sprmCIco:
        case SPRM::sprmCHps:
        case SPRM::sprmCHpsInc:
        case SPRM::sprmCHpsPos:
        case SPRM::sprmCHpsPosAdj:
            return 1;
        case SPRM::sprmCMajority:
            break; // Variable.
        case SPRM::sprmCIss:
            return 1;
        case SPRM::sprmCHpsNew50:
        case SPRM::sprmCHpsInc1:
            break; // Variable.
        case SPRM::sprmCHpsKern:
            return 2;
        case SPRM::sprmCMajority50:
            break; // Variable.
        case SPRM::sprmCHpsMul:
        case SPRM::sprmCCondHyhen:
            return 2;
        case SPRM::sprmCFSpec:
        case SPRM::sprmCFObj:
        case SPRM::sprmPicBrcl:
            return 1;
        case SPRM::sprmPicScale:
            break; // Variable.
        case SPRM::sprmPicBrcTop:
        case SPRM::sprmPicBrcLeft:
        case SPRM::sprmPicBrcBottom:
        case SPRM::sprmPicBrcRight:
            return 2;
        case SPRM::sprmSScnsPgn:
        case SPRM::sprmSiHeadingPgn:
            return 1;
        case SPRM::sprmSOlstAnm:
            break; // Variable.
        case SPRM::sprmSDxaColWidth:
        case SPRM::sprmSDxaColSpacing:
            return 3;
        case SPRM::sprmSFEvenlySpaced:
        case SPRM::sprmSFProtected:
            return 1;
        case SPRM::sprmSDmBinFirst:
        case SPRM::sprmSDmBinOther:
            return 2;
        case SPRM::sprmSBkc:
        case SPRM::sprmSFTitlePage:
            return 1;
        case SPRM::sprmSCcolumns:
        case SPRM::sprmSDxaColumns:
            return 2;
        case SPRM::sprmSFAutoPgn:
        case SPRM::sprmSNfcPgn:
            return 1;
        case SPRM::sprmSDyaPgn:
        case SPRM::sprmSDxaPgn:
            return 2;
        case SPRM::sprmSFPgnRestart:
        case SPRM::sprmSFEndnote:
        case SPRM::sprmSLnc:
        case SPRM::sprmSGprfIhdt:
            return 1;
        case SPRM::sprmSNLnnMod:
        case SPRM::sprmSDxaLnn:
        case SPRM::sprmSDyaHdrTop:
        case SPRM::sprmSDyaHdrBottom:
            return 2;
        case SPRM::sprmSLBetween:
        case SPRM::sprmSVjc:
            return 1;
        case SPRM::sprmSLnnMin:
        case SPRM::sprmSPgnStart:
            return 2;
        case SPRM::sprmSBOrientation:
        case SPRM::sprmSBCustomize:
            return 1;
        case SPRM::sprmSXaPage:
        case SPRM::sprmSYaPage:
        case SPRM::sprmSDxaLeft:
        case SPRM::sprmSDxaRight:
        case SPRM::sprmSDyaTop:
        case SPRM::sprmSDyaBottom:
        case SPRM::sprmSDzaGutter:
        case SPRM::sprmSDMPaperReq:
        case SPRM::sprmTJc:
        case SPRM::sprmTDxaLeft:
        case SPRM::sprmTDxaGapHalf:
            return 2;
        case SPRM::sprmTFCantSplit:
        case SPRM::sprmTTableHeader:
            return 1;
        case SPRM::sprmTTableBorders:
            return 12;
        case SPRM::sprmTDefTable10:
            return readU16(in) + 1;
        case SPRM::sprmTDyaRowHeight:
            return 2;
        case SPRM::sprmTDefTable:
            return readU16(in) + 1;
        case SPRM::sprmTDefTableShd:
            break; // Variable.
        case SPRM::sprmTTlp:
            return 4;
        case SPRM::sprmTSetBrc:
            return 5;
        case SPRM::sprmTInsert:
            return 4;
        case SPRM::sprmTDelete:
            return 2;
        case SPRM::sprmTDxaCol:
            return 4;
        case SPRM::sprmTMerge:
        case SPRM::sprmTSplit:
            return 2;
        case SPRM::sprmTSetBrc10:
            return 5;
        case SPRM::sprmTSetShd:
            return 4;
    }

    // Get length of variable size operand.
    return *in + 1;
}

} // namespace SPRM

// Returns -1 if this wasn't a PAP sprm and it returns the length
// of the applied sprm if it was successful
/*S16 PAP::applyPAPSPRM95( const U8 *ptr, const Style* style, OLEStreamReader* dataStream )
{
    U8 sprm = *ptr;
    ptr += sizeof( sprm );

    // Is it a PAP sprm?
    // which one? ;)
    switch ( sprm ) {
        case SPRM::sprmNoop:
            std::cerr << "Huh? Found a sprmNoop..." << std::endl;
            break;
        case SPRM::sprmPIstd:
            istd = readU16( ptr );
            break;
        case SPRM::sprmPIstdPermute:
            std::cerr << "Warning: sprmPIstdPermute not implemented" << std::endl;
            break;
        case SPRM::sprmPIncLvl:
            std::cerr << "Warning: sprmPIncLvl not implemented" << std::endl;
            break;
        case SPRM::sprmPJc:
            jc = *ptr;
            break;
        case SPRM::sprmPFSideBySide:
            fSideBySide = *ptr == 1;
            break;
        case SPRM::sprmPFKeep:
            fKeep = *ptr == 1;
            break;
        case SPRM::sprmPFKeepFollow:
            fKeepFollow = *ptr == 1;
            break;
        case SPRM::sprmPFPageBreakBefore:
            fPageBreakBefore = *ptr == 1;
            break;
        case SPRM::sprmPBrcl:
            brcl = *ptr;
            break;
        case SPRM::sprmPBrcp:
            brcp = *ptr;
            break;
        case SPRM::sprmPAnld:
            std::cerr << "Warning: sprmPAnld not implemented" << std::endl;
            break;
        case SPRM::sprmPNLvlAnm:
            nLvlAnm = *ptr;
            break;
        case SPRM::sprmPFNoLineNumb:
            fNoLnn = *ptr == 1;
            break;
        case SPRM::sprmPChgTabsPapx:
            std::cerr << "Warning: sprmPIncLvl not implemented" << std::endl;
            break;
        case SPRM::sprmPDxaRight:
            dxaRight = readS16( ptr );
            break;
        case SPRM::sprmPDxaLeft:
            dxaLeft = readS16( ptr );
            break;
        case SPRM::sprmPNest:
            dxaLeft += readS16( ptr );
            dxaLeft = dxaLeft < 0 ? 0 : dxaLeft;
            break;
        case SPRM::sprmPDxaLeft1:
            dxaLeft1 = readS16( ptr );
            break;
        case SPRM::sprmPDyaLine:
            std::cerr << "Warning: sprmPDyaLine not implemented" << std::endl;
            break;
        case SPRM::sprmPDyaBefore:
            dyaBefore = readU16( ptr );
            break;
        case SPRM::sprmPDyaAfter:
            dyaAfter = readU16( ptr );
            break;
        case SPRM::sprmPChgTabs:
            std::cerr << "Warning: sprmPChgTabs not implemented" << std::endl;
            break;
        case SPRM::sprmPFInTable:
            fInTable = *ptr == 1;
            break;
        case SPRM::sprmPFTtp:
            fTtp = *ptr == 1;
            break;
        case SPRM::sprmPDxaAbs:
            dxaAbs = readS16( ptr );
            break;
        case SPRM::sprmPDyaAbs:
            dyaAbs = readS16( ptr );
            break;
        case SPRM::sprmPDxaWidth:
            dxaWidth = readS16( ptr );
            break;
        case SPRM::sprmPPc:
            std::cerr << "Warning: sprmPPc not implemented" << std::endl;
            break;
        case SPRM::sprmPBrcTop10:
            std::cerr << "Warning: sprmPBrcTop10 doesn't make sense for Word 6" << std::endl;
            break;
        case SPRM::sprmPBrcLeft10:
            std::cerr << "Warning: sprmPBrcLeft10 doesn't make sense for Word 6" << std::endl;
            break;
        case SPRM::sprmPBrcBottom10:
            std::cerr << "Warning: sprmPBrcBottom10 doesn't make sense for Word 6" << std::endl;
            break;
        case SPRM::sprmPBrcRight10:
            std::cerr << "Warning: sprmPBrcRight10 doesn't make sense for Word 6" << std::endl;
            break;
        case SPRM::sprmPBrcBetween10:
            std::cerr << "Warning: sprmPBrcBetween10 doesn't make sense for Word 6" << std::endl;
            break;
        case SPRM::sprmPBrcBar10:
            std::cerr << "Warning: sprmPBrcBar10 doesn't make sense for Word 6" << std::endl;
            break;
        case SPRM::sprmPFromText10:
            std::cerr << "Warning: sprmPFromText10 doesn't make sense for Word 6" << std::endl;
            break;
        case SPRM::sprmPWr:
            wr = *ptr;
            break;
        case SPRM::sprmPBrcTop:
            brcTop.readPtr( ptr );
            break;
        case SPRM::sprmPBrcLeft:
            brcLeft.readPtr( ptr );
            break;
        case SPRM::sprmPBrcBottom:
            brcBottom.readPtr( ptr );
            break;
        case SPRM::sprmPBrcRight:
            brcRight.readPtr( ptr );
            break;
        case SPRM::sprmPBrcBetween:
            brcBetween.readPtr( ptr );
            break;
        case SPRM::sprmPBrcBar:
            brcBar.readPtr( ptr );
            break;
        case SPRM::sprmPFNoAutoHyph:
            fAutoHyph = *ptr == 1;
            break;
        case SPRM::sprmPWHeightAbs:
            std::cerr << "Warning: sprmPWHeightAbs not implemented" << std::endl;
            break;
        case SPRM::sprmPDcs:
            dcs.readPtr( ptr );
            break;
        case SPRM::sprmPShd:
            shd.readPtr( ptr );
            break;
        case SPRM::sprmPDyaFromText:
            dyaFromText = readS16( ptr );
            break;
        case SPRM::sprmPDxaFromText:
            dxaFromText = readS16( ptr );
            break;
        case SPRM::sprmPFLocked:
            fLocked = *ptr == 1;
            break;
        case SPRM::sprmPFWidowControl:
            fWidowControl = *ptr == 1;
            break;
        case SPRM::sprmPRuler:
            std::cerr << "Warning: sprmPRuler not implemented" << std::endl;
            break;
        default:
            std::cerr << "Warning: You're trying to apply a non PAP sprm to a PAP. Not neccessarly bad." << std::endl;
            return -1;  // tell the caller to try with something else (e.g. applying a TAP)...
    }
    return static_cast<S16>( SPRM::determineParameterLength( sprm, ptr ) ) + sizeof( sprm );  // length of the SPRM
}
*/

/*
// Returns -1 if this wasn't a CHP sprm and it returns the length
// of the applied sprm if it was successful
S16 CHP::applyCHPSPRM95( const U8* ptr, const Style* paragraphStyle, OLEStreamReader* dataStream )
{
    U8 sprm = *ptr;
    ptr += sizeof( sprm );

    // Is it a CHP sprm?
    // which one? ;)
    switch ( sprm ) {
        case SPRM::sprmNoop:
            std::cerr << "Huh? Found a sprmNoop..." << std::endl;
            break;
        case SPRM::sprmCFStrikeRM:
            fRMarkDel = *ptr == 1;
            break;
        case SPRM::sprmCFRMark:
            fRMark = *ptr == 1;
            break;
        case SPRM::sprmCFFldVanish:
            fFldVanish = *ptr == 1;
            break;
        case SPRM::sprmCPicLocation:
            std::cerr << "Warning: sprmCPicLocation not implemented" << std::endl;
            break;
        case SPRM::sprmCIbstRMark:
            ibstRMark = readS16( ptr );
            break;
        case SPRM::sprmCDttmRMark:
            dttmRMark.readPtr( ptr );
            break;
        case SPRM::sprmCFData:
            fData = *ptr == 1;
            break;
	case SPRM::sprmCRMReason:
            idslRMReason = readU16( ptr );
            break;
	case SPRM::sprmCChse:
            std::cerr << "Warning: sprmCChse not implemented" << std::endl;
            break;
        case SPRM::sprmCSymbol:
            std::cerr << "Warning: sprmCSymbol not implemented" << std::endl;
            break;
        case SPRM::sprmCFOle2:
            fOle2 = *ptr == 1;
            break;
        case SPRM::sprmCIstd:
            istd = readS16( ptr );
            break;
        case SPRM::sprmCIstdPermute:
            std::cerr << "Warning: sprmCIstdPermute not implemented" << std::endl;
            break;
        case SPRM::sprmCDefault:
            std::cerr << "Warning: sprmCDefault not implemented" << std::endl;
            break;
        case SPRM::sprmCPlain:
            std::cerr << "Warning: sprmCPlain not implemented" << std::endl;
            break;
        case SPRM::sprmCFBold:
            if ( *ptr < 128 )
                fBold = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFBold extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFItalic:
            if ( *ptr < 128 )
                fItalic = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFItalic extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFStrike:
            if ( *ptr < 128 )
                fStrike = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFStrike extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFOutline:
            if ( *ptr < 128 )
                fOutline = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFOutline extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFShadow:
            if ( *ptr < 128 )
                fShadow = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFShadow extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFSmallCaps:
            if ( *ptr < 128 )
                fSmallCaps = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFSmallCaps extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFCaps:
            if ( *ptr < 128 )
                fCaps = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFCaps extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFVanish:
            if ( *ptr < 128 )
                fVanish = *ptr == 1;
            else
                std::cerr << "Warning: sprmCFVanish extended not implemented" << std::endl;
            break;
        case SPRM::sprmCFtc:
            ftc = readS16( ptr );
            break;
        case SPRM::sprmCKul:
            kul = *ptr;
            break;
        case SPRM::sprmCSizePos:
            std::cerr << "Warning: sprmCSizePos not implemented" << std::endl;
            break;
        case SPRM::sprmCDxaSpace:
            dxaSpace = readS16( ptr );
            break;
        case SPRM::sprmCLid:
            std::cerr << "Warning: sprmCLid not implemented" << std::endl;
            break;
        case SPRM::sprmCIco:
            ico = *ptr;
            break;
        case SPRM::sprmCHps:
            hps = readU16( ptr );
            break;
        case SPRM::sprmCHpsInc:
            std::cerr << "Warning: sprmCHpsInc not implemented" << std::endl;
            break;
        case SPRM::sprmCHpsPos:
            hpsPos = readS16( ptr );
            break;
        case SPRM::sprmCHpsPosAdj:
            std::cerr << "Warning: sprmCHpsPosAdj not implemented" << std::endl;
            break;
        case SPRM::sprmCMajority:
            std::cerr << "Warning: sprmCMajority not implemented" << std::endl;
            break;
        case SPRM::sprmCIss:
            iss = *ptr;
            break;
        case SPRM::sprmCHpsNew50:
            std::cerr << "Warning: sprmCHpsNew50 not implemented" << std::endl;
            break;
        case SPRM::sprmCHpsInc1:
            std::cerr << "Warning: sprmCHpsInc1 not implemented" << std::endl;
            break;
        case SPRM::sprmCHpsKern:
            hpsKern = readU16( ptr );
            break;
        case SPRM::sprmCMajority50:
            std::cerr << "Warning: sprmCMajority50 not implemented" << std::endl;
            break;
        case SPRM::sprmCHpsMul:
            std::cerr << "Warning: sprmCHpsMul not implemented" << std::endl;
            break;
        case SPRM::sprmCCondHyhen:
            std::cerr << "Warning: sprmCCondHyhen not implemented" << std::endl;
            break;
        case SPRM::sprmCFSpec:
            fSpec = *ptr == 1;
            break;
        case SPRM::sprmCFObj:
            fObj = *ptr == 1;
            break;
        default:
            std::cerr << "Warning: You're trying to apply a non CHP sprm to a CHP. Not neccessarly bad." << std::endl;
            return -1;
    }
    return static_cast<S16>( SPRM::determineParameterLength( sprm, ptr ) ) + sizeof( sprm ); // length of the SPRM
}
*/

/*
// Returns -1 if this wasn't a PICF sprm and it returns the length
// of the applied sprm if it was successful
S16 PICF::applyPICFSPRM95( const U8* ptr, const Style* style, OLEStreamReader* dataStream )
{
    U8 sprm = *ptr;
    ptr += sizeof( sprm );

    // Is it a PICF sprm?
    // which one? ;)
    switch ( sprm ) {
        case SPRM::sprmNoop:
            std::cerr << "Huh? Found a sprmNoop..." << std::endl;
            break;
        case SPRM::sprmPicBrcl:
            brcl = *ptr;
            break;
        case SPRM::sprmPicScale:
            std::cerr << "Warning: sprmPicScale not implemented" << std::endl;
            break;
        case SPRM::sprmPicBrcTop:
            brcTop.readPtr( ptr );
            break;
        case SPRM::sprmPicBrcLeft:
            brcLeft.readPtr( ptr );
            break;
        case SPRM::sprmPicBrcBottom:
            brcBottom.readPtr( ptr );
            break;
        case SPRM::sprmPicBrcRight:
            brcRight.readPtr( ptr );
            break;
        default:
            std::cerr << "Warning: You're trying to apply a non PICF sprm to a PICF." << std::endl;
            return -1;
    }
    return static_cast<S16>( SPRM::determineParameterLength( sprm, ptr ) ) + sizeof( sprm ); // length of the SPRM
}
*/
/*
// Returns -1 if this wasn't a SEP sprm and it returns the length
// of the applied sprm if it was successful
S16 SEP::applySEPSPRM95( const U8* ptr, const Style* style, OLEStreamReader* dataStream )
{
    U8 sprm = *ptr;
    ptr += sizeof( sprm );

    // Is it a SEP sprm?
    // which one? ;)
    switch ( sprm ) {
        case SPRM::sprmNoop:
            std::cerr << "Huh? Found a sprmNoop..." << std::endl;
            break;
        case SPRM::sprmSScnsPgn:
            cnsPgn = *ptr;
            break;
        case SPRM::sprmSiHeadingPgn:
            iHeadingPgn = *ptr;
            break;
        case SPRM::sprmSOlstAnm:
            std::cerr << "Warning: sprmSOlstAnm not implemented" << std::endl;
            break;
        case SPRM::sprmSDxaColWidth:
            std::cerr << "Warning: sprmSDxaColWidth not implemented" << std::endl;
            break;
        case SPRM::sprmSDxaColSpacing:
            std::cerr << "Warning: sprmSDxaColSpacing not implemented" << std::endl;
            break;
        case SPRM::sprmSFEvenlySpaced:
            fEvenlySpaced = *ptr == 1;
            break;
        case SPRM::sprmSFProtected:
            fUnlocked = *ptr == 1;
            break;
        case SPRM::sprmSDmBinFirst:
            dmBinFirst = readU16( ptr );
            break;
        case SPRM::sprmSDmBinOther:
            dmBinOther = readU16( ptr );
            break;
        case SPRM::sprmSBkc:
            bkc = *ptr;
            break;
        case SPRM::sprmSFTitlePage:
            fTitlePage = *ptr == 1;
            break;
        case SPRM::sprmSCcolumns:
            ccolM1 = readS16( ptr );
            break;
        case SPRM::sprmSDxaColumns:
            dxaColumns = readS16( ptr );
            break;
        case SPRM::sprmSFAutoPgn:
            fAutoPgn = *ptr == 1;
            break;
        case SPRM::sprmSNfcPgn:
            nfcPgn = *ptr;
            break;
        case SPRM::sprmSDyaPgn:
            dyaPgn = readS16( ptr );
            break;
        case SPRM::sprmSDxaPgn:
            dxaPgn = readS16( ptr );
            break;
        case SPRM::sprmSFPgnRestart:
            fPgnRestart = *ptr == 1;
            break;
        case SPRM::sprmSFEndnote:
            fEndNote = *ptr == 1;
            break;
        case SPRM::sprmSLnc:
            lnc = *ptr;
            break;
        case SPRM::sprmSGprfIhdt:
            grpfIhdt = *ptr;
            break;
        case SPRM::sprmSNLnnMod:
            nLnnMod = readU16( ptr );
            break;
        case SPRM::sprmSDxaLnn:
            dxaLnn = readS16( ptr );
            break;
        case SPRM::sprmSDyaHdrTop:
            dyaHdrTop = readU16( ptr );
            break;
        case SPRM::sprmSDyaHdrBottom:
            dyaHdrBottom = readU16( ptr );
            break;
        case SPRM::sprmSLBetween:
            fLBetween = *ptr == 1;
            break;
        case SPRM::sprmSVjc:
            vjc = *ptr;
            break;
        case SPRM::sprmSLnnMin:
            lnnMin = readS16( ptr );
            break;
        case SPRM::sprmSPgnStart:
            pgnStart = readU16( ptr );
            break;
        case SPRM::sprmSBOrientation:
            dmOrientPage = *ptr;
            break;
        case SPRM::sprmSBCustomize:
            std::cerr << "Warning: sprmSBCustomize not implemented" << std::endl;
            break;
        case SPRM::sprmSXaPage:
            xaPage = readU16( ptr );
            break;
        case SPRM::sprmSYaPage:
            yaPage = readU16( ptr );
            break;
        case SPRM::sprmSDxaLeft:
            dxaLeft = readU16( ptr );
            break;
        case SPRM::sprmSDxaRight:
            dxaRight = readU16( ptr );
            break;
        case SPRM::sprmSDyaTop:
            dyaTop = readU16( ptr );
            break;
        case SPRM::sprmSDyaBottom:
            dyaBottom = readU16( ptr );
            break;
        case SPRM::sprmSDzaGutter:
            dzaGutter = readU16( ptr );
            break;
        case SPRM::sprmSDMPaperReq:
            dmPaperReq = readU16( ptr );
            break;
        default:
            std::cerr << "Warning: You're trying to apply a non SEP sprm to a SEP." << std::endl;
            return -1;
    }
    return static_cast<S16>( SPRM::determineParameterLength( sprm, ptr ) ) + sizeof( sprm ); // length of the SPRM
}
*/

/*
// Returns -1 if this wasn't a TAP sprm and it returns the length
// of the applied sprm if it was successful
S16 TAP::applyTAPSPRM95( const U8* ptr, const Style* style, OLEStreamReader* dataStream )
{
    U8 sprm = *ptr;
    ptr += sizeof( sprm );

    // Is it a TAP sprm?
    // which one? ;)
    switch ( sprm ) {
        case SPRM::sprmNoop:
            std::cerr << "Huh? Found a sprmNoop..." << std::endl;
            break;
        case SPRM::sprmTJc:
            jc = readS16( ptr );
            break;
        case SPRM::sprmTDxaLeft:
            std::cerr << "Warning: sprmTDxaLeft not implemented" << std::endl;
            break;
        case SPRM::sprmTDxaGapHalf:
            std::cerr << "Warning: sprmTDxaGapHalf not implemented" << std::endl;
            break;
        case SPRM::sprmTFCantSplit:
            fCantSplit = *ptr == 1;
            break;
        case SPRM::sprmTTableHeader:
            fTableHeader = *ptr == 1;
            break;
        case SPRM::sprmTTableBorders:
            std::cerr << "Warning: sprmTTableBorders not implemented" << std::endl;
            break;
        case SPRM::sprmTDefTable10:
            std::cerr << "Warning: sprmTDefTable10 not implemented" << std::endl;
            break;
        case SPRM::sprmTDyaRowHeight:
            dyaRowHeight = readS16( ptr );
            break;
        case SPRM::sprmTDefTable:
            std::cerr << "Warning: sprmTDefTable not implemented" << std::endl;
            break;
        case SPRM::sprmTDefTableShd:
            std::cerr << "Warning: sprmTDefTableShd not implemented" << std::endl;
            break;
        case SPRM::sprmTTlp:
            tlp.readPtr( ptr );
            break;
        case SPRM::sprmTSetBrc:
            std::cerr << "Warning: sprmTSetBrc not implemented" << std::endl;
            break;
        case SPRM::sprmTInsert:
            std::cerr << "Warning: sprmTInsert not implemented" << std::endl;
            break;
        case SPRM::sprmTDelete:
            std::cerr << "Warning: sprmTDelete not implemented" << std::endl;
            break;
        case SPRM::sprmTDxaCol:
            std::cerr << "Warning: sprmTDxaCol not implemented" << std::endl;
            break;
        case SPRM::sprmTMerge:
            std::cerr << "Warning: sprmTMerge not implemented" << std::endl;
            break;
        case SPRM::sprmTSplit:
            std::cerr << "Warning: sprmTSplit not implemented" << std::endl;
            break;
        case SPRM::sprmTSetBrc10:
            std::cerr << "Warning: sprmTSetBrc10 not implemented" << std::endl;
            break;
        case SPRM::sprmTSetShd:
            std::cerr << "Warning: sprmTSetShd not implemented" << std::endl;
            break;
        default:
            std::cerr << "Warning: You're trying to apply a non TAP sprm to a TAP. Not neccessarly bad." << std::endl;
            return -1;
    }
    return static_cast<S16>( SPRM::determineParameterLength( sprm, ptr ) ) + sizeof( sprm ); // length of the SPRM
}
*/

} // namespace Word95
} // namespace wvWare
